@@ -11,8 +11,9 @@ import pytest
 import pandas.util._test_decorators as td
 
 from pandas import DataFrame
-import pandas.util.testing as tm
+import pandas._testing as tm
 
+from pandas.io.feather_format import read_feather
 from pandas.io.parsers import read_csv
 
 
@@ -48,14 +49,14 @@ def check_compressed_urls(salaries_table, compression, extension, mode, engine):
 @pytest.fixture
 def tips_df(datapath):
     """DataFrame with the tips dataset."""
-    return read_csv(datapath("io", "parser", "data", "tips.csv"))
+    return read_csv(datapath("io", "data", "csv", "tips.csv"))
 
 
 @pytest.mark.usefixtures("s3_resource")
 @td.skip_if_not_us_locale()
 class TestS3:
+    @td.skip_if_no("s3fs")
     def test_parse_public_s3_bucket(self, tips_df):
-        pytest.importorskip("s3fs")
 
         # more of an integration test due to the not-public contents portion
         # can probably mock this though.
@@ -159,14 +160,41 @@ class TestS3:
             assert not df.empty
             tm.assert_frame_equal(tips_df.iloc[:10], df)
 
-    def test_s3_fails(self):
+    def test_read_s3_fails(self):
         with pytest.raises(IOError):
             read_csv("s3://nyqpug/asdf.csv")
 
         # Receive a permission error when trying to read a private bucket.
         # It's irrelevant here that this isn't actually a table.
         with pytest.raises(IOError):
-            read_csv("s3://cant_get_it/")
+            read_csv("s3://cant_get_it/file.csv")
+
+    def test_write_s3_csv_fails(self, tips_df):
+        # GH 32486
+        # Attempting to write to an invalid S3 path should raise
+        import botocore
+
+        # GH 34087
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/error-handling.html
+        # Catch a ClientError since AWS Service Errors are defined dynamically
+        error = (FileNotFoundError, botocore.exceptions.ClientError)
+
+        with pytest.raises(error, match="The specified bucket does not exist"):
+            tips_df.to_csv("s3://an_s3_bucket_data_doesnt_exit/not_real.csv")
+
+    @td.skip_if_no("pyarrow")
+    def test_write_s3_parquet_fails(self, tips_df):
+        # GH 27679
+        # Attempting to write to an invalid S3 path should raise
+        import botocore
+
+        # GH 34087
+        # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/error-handling.html
+        # Catch a ClientError since AWS Service Errors are defined dynamically
+        error = (FileNotFoundError, botocore.exceptions.ClientError)
+
+        with pytest.raises(error, match="The specified bucket does not exist"):
+            tips_df.to_parquet("s3://an_s3_bucket_data_doesnt_exit/not_real.parquet")
 
     def test_read_csv_handles_boto_s3_object(self, s3_resource, tips_file):
         # see gh-16135
@@ -184,8 +212,9 @@ class TestS3:
 
     def test_read_csv_chunked_download(self, s3_resource, caplog):
         # 8 MB, S3FS usees 5MB chunks
+        import s3fs
+
         df = DataFrame(np.random.randn(100000, 4), columns=list("abcd"))
-        buf = BytesIO()
         str_buf = StringIO()
 
         df.to_csv(str_buf)
@@ -194,12 +223,25 @@ class TestS3:
 
         s3_resource.Bucket("pandas-test").put_object(Key="large-file.csv", Body=buf)
 
-        with caplog.at_level(logging.DEBUG, logger="s3fs.core"):
+        # Possibly some state leaking in between tests.
+        # If we don't clear this cache, we saw `GetObject operation: Forbidden`.
+        # Presumably the s3fs instance is being cached, with the directory listing
+        # from *before* we add the large-file.csv in the pandas-test bucket.
+        s3fs.S3FileSystem.clear_instance_cache()
+
+        with caplog.at_level(logging.DEBUG, logger="s3fs"):
             read_csv("s3://pandas-test/large-file.csv", nrows=5)
             # log of fetch_range (start, stop)
-            assert (0, 5505024) in {x.args[-2:] for x in caplog.records}
+            assert (0, 5505024) in (x.args[-2:] for x in caplog.records)
 
     def test_read_s3_with_hash_in_key(self, tips_df):
         # GH 25945
         result = read_csv("s3://pandas-test/tips#1.csv")
         tm.assert_frame_equal(tips_df, result)
+
+    @td.skip_if_no("pyarrow")
+    def test_read_feather_s3_file_path(self, feather_file):
+        # GH 29055
+        expected = read_feather(feather_file)
+        res = read_feather("s3://pandas-test/simple_dataset.feather")
+        tm.assert_frame_equal(expected, res)
